// -*- C++ -*-
/*!
 * @file  StereoCameraCalibration.cpp
 * @brief Stereo Camera Calibration
 * @date $Date$
 * @author Hiroaki Yaguchi, JSK
 *
 * $Id$
 */

#include "StereoCameraCalibration.h"

// Module specification
// <rtc-template block="module_spec">
static const char* stereocameracalibration_spec[] =
  {
    "implementation_id", "StereoCameraCalibration",
    "type_name",         "StereoCameraCalibration",
    "description",       "Stereo Camera Calibration",
    "version",           "1.0.0",
    "vendor",            "Hiroaki Yaguchi, JSK",
    "category",          "Image Processing",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.board_width", "7",
    "conf.default.board_height", "5",
    "conf.default.square_size", "30.0",
    "conf.default.stop_count", "6",
    //"conf.default.pixel_size", "0.0056",
    "conf.default.min_image_count", "8",
    "conf.default.continuous_mode", "false",
    // Widget
    "conf.__widget__.board_width", "text",
    "conf.__widget__.board_height", "text",
    "conf.__widget__.square_size", "text",
    //"conf.__widget__.pixel_size", "text",
    "conf.__widget__.min_image_count", "text",
    "conf.__widget__.continuous_mode", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
StereoCameraCalibration::StereoCameraCalibration(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_LeftImageIn("LeftImage", m_LeftImage),
    m_RightImageIn("RightImage", m_RightImage),
    m_LeftImageOut("SelectedLeftImage", m_SelectedLeftImage),
    m_RightImageOut("SelectedRightImage", m_SelectedRightImage)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
StereoCameraCalibration::~StereoCameraCalibration()
{
}



RTC::ReturnCode_t StereoCameraCalibration::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("LeftImage", m_LeftImageIn);
  addInPort("RightImage", m_RightImageIn);
  
  // Set OutPort buffer

  addOutPort("SelectedLeftImage", m_LeftImageOut);
  addOutPort("SelectedRightImage", m_RightImageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  bindParameter("board_width", m_board_width, "7");
  bindParameter("board_height", m_board_height, "5");
  bindParameter("square_size", m_square_size, "30.0");
  bindParameter("stop_count", m_stop_count, "6");
  //bindParameter("pixel_size", m_pixel_size, "0.0056");
  bindParameter("min_image_count", m_min_image_count, "8");
  bindParameter("continuous_mode", m_continuous_mode, "false");

  namedWindow("left");
  namedWindow("right");

  waitKey(10);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t StereoCameraCalibration::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t StereoCameraCalibration::onActivated(RTC::UniqueId ec_id)
{
  boardSize=Size(m_board_width,m_board_height);
  isnew_left=isnew_right=false;
  squareSize=m_square_size;
  imagecount=0;
  imagePoints[0].clear();
  imagePoints[1].clear();
  imageSize=Size();

  objectPointsMat.create(m_board_width*m_board_height,3,CV_32F);
  imagePointsMat.create(m_board_width*m_board_height,2,CV_32F);

  prev_corners_left.clear();
  prev_corners_right.clear();
  boardPoses.clear();
  stopcount=0;

  for(int i=0;i<m_board_height;i++){
    int ii=i*m_board_width;
    for(int j=0;j<m_board_width;j++){
      objectPointsMat.at<float>(ii+j,0)=i*squareSize;
      objectPointsMat.at<float>(ii+j,1)=j*squareSize;
      objectPointsMat.at<float>(ii+j,2)=0.0;

      prev_corners_left.push_back(Point2f(0.0,0.0));
      prev_corners_right.push_back(Point2f(0.0,0.0));
    }
  }

  continuousp=(m_continuous_mode=="true");

  return RTC::RTC_OK;
}


RTC::ReturnCode_t StereoCameraCalibration::onDeactivated(RTC::UniqueId ec_id)
{
  if(imagecount>=m_min_image_count){
    cout << "calibration start" << endl;
    doCalibration();
  }
  return RTC::RTC_OK;
}


RTC::ReturnCode_t StereoCameraCalibration::onExecute(RTC::UniqueId ec_id)
{
  bool isnew=false;

  if(m_LeftImageIn.isNew()){
    m_LeftImageIn.read();
    convertToCV(m_LeftImage,cvleftimage);
    convertToCV(m_LeftImage,cvleftimageOut);
    isnew_left=true;
  }
  if(m_RightImageIn.isNew()){
    m_RightImageIn.read();
    convertToCV(m_RightImage,cvrightimage);
    convertToCV(m_RightImage,cvrightimageOut);
    isnew_right=true;
  }

  isnew = isnew_left && isnew_right;

  if(isnew){
    if(imageSize==Size()) imageSize=cvleftimage.size();
    Mat cvleftcolor,cvrightcolor,cvleftgray,cvrightgray;

    if(cvleftimage.type()==CV_8UC3){
      cvleftcolor=cvleftimage;
      cvtColor(cvleftimage,cvleftgray,CV_BGR2GRAY);
    }else{
      cvtColor(cvleftimage,cvleftcolor,CV_GRAY2BGR);
      cvleftgray=cvleftimage;
    }

    if(cvrightimage.type()==CV_8UC3){
      cvrightcolor=cvrightimage;
      cvtColor(cvrightimage,cvrightgray,CV_BGR2GRAY);
    }else{
      cvtColor(cvrightimage,cvrightcolor,CV_GRAY2BGR);
      cvrightgray=cvrightimage;
    }

    vector<Point2f> corners_left,corners_right;
    if(findChessboard(cvleftgray,cvleftcolor,corners_left) &&
       findChessboard(cvrightgray,cvrightcolor,corners_right)){

      cout << "onExecute start." << endl;

      checkDirection(corners_left);
      checkDirection(corners_right);

      cornerSubPix(cvleftgray,corners_left,
		   Size(11,11),Size(-1,-1),
		   TermCriteria(CV_TERMCRIT_ITER+CV_TERMCRIT_EPS,30, 0.01));
      cornerSubPix(cvrightgray,corners_right,
		   Size(11,11),Size(-1,-1),
		   TermCriteria(CV_TERMCRIT_ITER+CV_TERMCRIT_EPS,30, 0.01));

      if(continuousp){
	stopcount=m_stop_count;
      }else{
	bool cml,cmr;
	cml=checkMotion(prev_corners_left,corners_left);
	cmr=checkMotion(prev_corners_right,corners_right);
	prev_corners_left.assign(corners_left.begin(),corners_left.end());
	prev_corners_right.assign(corners_right.begin(),corners_right.end());

	if(cml && cmr)
	  stopcount++;
	else
	  stopcount=0;
      }



      if(stopcount>=m_stop_count){
	vector<float> cbpose;
	estimatePose(corners_left,cbpose);

	bool writep=false;
	if(imagecount>0){
	  if(checkMotion3D(boardPoses[imagecount-1],cbpose) || continuousp){
	    imagePoints[1].push_back(corners_right);
	    imagePoints[0].push_back(corners_left);
	    imagecount++;
	    boardPoses.push_back(cbpose);
	    cout << " [onExecute]: saved." << endl;
	    stopcount=0;
	    writep=true;
	  }
	}else{
	  imagePoints[1].push_back(corners_right);
	  imagePoints[0].push_back(corners_left);
	  imagecount++;
	  boardPoses.push_back(cbpose);
	  cout << " [onExecute]: saved." << endl;
	  stopcount=0;
	  writep=true;
	}


	if( writep==true){
	  convertToRTM(cvrightimageOut,m_SelectedRightImage);
	  convertToRTM(cvleftimageOut,m_SelectedLeftImage);
	  if(imagecount==1){
	    m_SelectedLeftImage.tm.sec=m_LeftImage.tm.sec;
	    m_SelectedLeftImage.tm.nsec=m_LeftImage.tm.nsec;
	    m_SelectedRightImage.tm.sec=m_RightImage.tm.sec;
	    m_SelectedRightImage.tm.nsec=m_RightImage.tm.nsec;
	  } else {
	    m_SelectedLeftImage.tm.sec+=1;
	    m_SelectedRightImage.tm.sec+=1;
	  }
	  m_RightImageOut.write();
	  m_LeftImageOut.write();
	}

      }
      cout << "onExecute end." << endl;
    }

    stringstream ss;
    ss << imagecount;
    putText(cvleftcolor,ss.str(),Size(50,50),FONT_HERSHEY_SIMPLEX,2.0,Scalar(0,0,255),3);
    putText(cvrightcolor,ss.str(),Size(50,50),FONT_HERSHEY_SIMPLEX,2.0,Scalar(0,0,255),3);

    imshow("left",cvleftcolor);
    imshow("right",cvrightcolor);
    waitKey(10);

    isnew_left=isnew_right=false;
  }

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t StereoCameraCalibration::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StereoCameraCalibration::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


bool StereoCameraCalibration::findChessboard(Mat &img,Mat &cimg,vector<Point2f> &corners){
  bool found=findChessboardCorners(img, boardSize, corners,
				   CV_CALIB_CB_ADAPTIVE_THRESH | CV_CALIB_CB_NORMALIZE_IMAGE | CV_CALIB_CB_FAST_CHECK);
  drawChessboardCorners(cimg, boardSize, corners, found);

  return found;
}

bool StereoCameraCalibration::checkMotion(vector<Point2f> &prev,vector<Point2f> &curr){
  float sum_dist=0.0;

  //cout << (prev.size() == boardSize.height * boardSize.width) << endl;
  //cout << (prev.size() == curr.size()) << endl;

  for(int i=0;i<prev.size();i++){
    Point2f pp=curr[i]-prev[i];
    sum_dist+=sqrt(pp.dot(pp));
  }
  //return (sum_dist/prev.size()) > 100.0;
  cout << " [checkMotion]: distance = " << (sum_dist/prev.size()) << endl;
  return (sum_dist/prev.size()) < 10.0;
}

void StereoCameraCalibration::estimatePose(vector<Point2f> &points,vector<float> &pose){
  // calibration board pose estimation

  for(int i=0;i<points.size();i++){
    imagePointsMat.at<float>(i,0)=points[i].x;
    imagePointsMat.at<float>(i,1)=points[i].y;
  }

  CvMat *cvintmat=cvCreateMat(3,3,CV_32F);
  cvSetZero(cvintmat);
  cvintmat->data.fl[0]=1.0; // focal
  cvintmat->data.fl[4]=1.0; // focal
  cvintmat->data.fl[8]=1.0;

  CvMat *cvrvec,*cvtvec;
  cvrvec=cvCreateMat(3,1,CV_32F);
  cvtvec=cvCreateMat(3,1,CV_32F);

  CvMat *dist=cvCreateMat(5,1,CV_32F);
  cvSetZero(dist);

  CvMat cvobjectpointsmat=objectPointsMat;
  CvMat cvimagepointsmat=imagePointsMat;

  cvFindExtrinsicCameraParams2(&cvobjectpointsmat,
			       &cvimagepointsmat,
			       cvintmat,dist,cvrvec,cvtvec);

  pose.clear();
  pose.resize(6);
  for(int i=0;i<3;i++){
    pose[i]=cvtvec->data.fl[i];
    pose[i+3]=cvrvec->data.fl[i];
  }

  cout << " [estimatePose]: translation = "
       << pose[0] << ", "
       << pose[1] << ", "
       << pose[2] <<endl;
  cout << " [estimatePose]: rotation = "
       << pose[3] << ", "
       << pose[4] << ", "
       << pose[5] <<endl;

  cvReleaseMat(&cvintmat);
  cvReleaseMat(&cvrvec);
  cvReleaseMat(&cvtvec);
  cvReleaseMat(&dist);
}

bool StereoCameraCalibration::checkMotion3D(vector<float> &prev_pose,vector<float> &curr_pose){

  cout << " [checkMotion3D]: (prev) translation = "
       << prev_pose[0] << ", "
       << prev_pose[1] << ", "
       << prev_pose[2] <<endl;
  cout << " [checkMotion3D]: (prev) rotation = "
       << prev_pose[3] << ", "
       << prev_pose[4] << ", "
       << prev_pose[5] <<endl;
  cout << " [checkMotion3D]: (curr) translation = "
       << curr_pose[0] << ", "
       << curr_pose[1] << ", "
       << curr_pose[2] <<endl;
  cout << " [checkMotion3D]: (curr) rotation = "
       << curr_pose[3] << ", "
       << curr_pose[4] << ", "
       << curr_pose[5] <<endl;

  // check transration
  float dist=0.0;
  for(int i=0;i<3;i++){
    float dd=prev_pose[i]-curr_pose[i];
    dist+=dd*dd;
  }
  dist=sqrt(dist);

  // check rotation
  Mat rv0,rv1,rm0,rm1,rv,rm;
  rv.create(1,3,CV_32F);
  rv0.create(1,3,CV_32F);
  rv1.create(1,3,CV_32F);
  rm.create(3,3,CV_32F);
  rm0.create(3,3,CV_32F);
  rm1.create(3,3,CV_32F);

  for(int i=0;i<3;i++){
    rv0.at<float>(i)=prev_pose[3+i];
    rv1.at<float>(i)=curr_pose[3+i];
  }
  Rodrigues(rv0,rm0);
  Rodrigues(rv1,rm1);
  rm=rm0.t()*rm1;
  Rodrigues(rm,rv);
  float theta=sqrt(rv.dot(rv));

  //
  cout << " [checkMotion3D]: distance = " << dist << endl;
  cout << " [checkMotion3D]: theta = " << theta << " [rad]" << endl;
  cout << " [checkMotion3D]: theta = " << theta/M_PI*180.0 << " [deg]" << endl;

  return (theta>(10.0*M_PI/180.0));
}


bool StereoCameraCalibration::checkDirection(vector<Point2f> &corners){
  int ncorners=corners.size();
  /*
  cout << "0:  " << corners[0] << endl;
  cout << boardSize.width << ":  " << corners[boardSize.width] << endl;
  cout << ncorners-boardSize.width << ": " << corners[ncorners-boardSize.width] << endl;
  cout << ncorners-1 << ": " << corners[ncorners-1] << endl;
  */

  Point2f di=corners[ncorners-1]-corners[0];

  cout << " [checkDirection]: Direction X : " << ((di.x > 0.0) ? "left to right" : "right to left") << endl;
  cout << " [checkDirection]: Direction Y : " << ((di.y > 0.0) ? "top to down" : "down to top") << endl;

  if((di.x < 0.0) && (di.y < 0.0)){
    cout << " [checkDirection]: reverse corners..." << endl;
    reverse(corners.begin(),corners.end());
  }

  return true;
}

bool StereoCameraCalibration::doCalibration(){
  // this function is based on OpenCV sample code
  // samples/cpp/stereo_calib.cpp
  int i,j,k;
  objectPoints.clear();
  objectPoints.resize(imagecount);
  for( i = 0; i < imagecount; i++ )
    {
      objectPoints[i].clear();
      for( j = 0; j < boardSize.height; j++ )
	for( k = 0; k < boardSize.width; k++ )
	  objectPoints[i].push_back(Point3f(j*squareSize, k*squareSize, 0));
    }

  Mat cameraMatrix[2], distCoeffs[2];
  cameraMatrix[0] = Mat::eye(3, 3, CV_64F);
  cameraMatrix[1] = Mat::eye(3, 3, CV_64F);
  Mat R, T, E, F;

  cout << "stereoCalibrate : input" << endl;
  cout << "object points :" << endl;
  for( i = 0; i < imagecount; i++ )
    {
      cout << objectPoints[i] << endl;
    }
  //cout << "image points[0] :" << imagePoints[0] << endl;
  //cout << "image points[1] :" << imagePoints[1] << endl;
  for( k = 0; k < 2; k++ )
    {
      cout << "image points["<< k<<"] :" << endl;
      for( i = 0; i < imagecount; i++ )
	{
	  int npt = (int)imagePoints[k][i].size();
	  cout << i << ":" << endl;
	  for( j = 0; j < npt; j++ )
	    {
	      cout << imagePoints[k][i][j] << endl;
	    }
	}
    }
  cout << "camera matrix[0] :" << cameraMatrix[0] << endl;
  cout << "camera matrix[1] :" << cameraMatrix[1] << endl;
  cout << "dist coeffs[0] :" << distCoeffs[0] << endl;
  cout << "dist coeffs[1] :" << distCoeffs[1] << endl;
  cout << "image size :" << imageSize.width  << "x" << imageSize.height << endl;
    
  double rms = stereoCalibrate(objectPoints, imagePoints[0], imagePoints[1],
			       cameraMatrix[0], distCoeffs[0],
			       cameraMatrix[1], distCoeffs[1],
			       imageSize, R, T, E, F,
			       TermCriteria(CV_TERMCRIT_ITER+CV_TERMCRIT_EPS, 100, 1e-5),
			       CV_CALIB_FIX_ASPECT_RATIO +
			       //CV_CALIB_ZERO_TANGENT_DIST +
			       //CV_CALIB_FIX_PRINCIPAL_POINT +
			       CV_CALIB_SAME_FOCAL_LENGTH +
			       CV_CALIB_RATIONAL_MODEL +
			       CV_CALIB_FIX_K3 + CV_CALIB_FIX_K4 + CV_CALIB_FIX_K5 + CV_CALIB_FIX_K6);
  cout << "done with RMS error=" << rms << endl;

  cout << "stereoCalibrate : output" << endl;
  cout << "camera matrix[0] :" << cameraMatrix[0] << endl;
  cout << "camera matrix[1] :" << cameraMatrix[1] << endl;
  cout << "dist coeffs[0] :" << distCoeffs[0] << endl;
  cout << "dist coeffs[1] :" << distCoeffs[1] << endl;
  cout << "R :" << R << endl;
  cout << "T :" << T << endl;
  cout << "E :" << E << endl;
  cout << "F :" << F << endl;

  double fx,fy,cx,cy;

  /*
  cout << "intrinsic params: " << endl;
  cout << cameraMatrix[0] << endl;
  cout << cameraMatrix[1] << endl;
  */

  fx=cameraMatrix[0].at<double>(0,0);
  fy=cameraMatrix[0].at<double>(1,1);
  cx=cameraMatrix[0].at<double>(0,2);
  cy=cameraMatrix[0].at<double>(1,2);
  cout << "intrinsic params (Left):" << endl;
  //cout << "  fx: " << fx * m_pixel_size << endl;
  //cout << "  fy: " << fy * m_pixel_size << endl;
  cout << "  fx: " << fx << endl;
  cout << "  fy: " << fy << endl;
  cout << "  cx: " << cx << endl;
  cout << "  cy: " << cy << endl;

  fx=cameraMatrix[1].at<double>(0,0);
  fy=cameraMatrix[1].at<double>(1,1);
  cx=cameraMatrix[1].at<double>(0,2);
  cy=cameraMatrix[1].at<double>(1,2);
  cout << "intrinsic params (Right):" << endl;
  //cout << "  fx: " << fx * m_pixel_size << endl;
  //cout << "  fy: " << fy * m_pixel_size << endl;
  cout << "  fx: " << fx << endl;
  cout << "  fy: " << fy << endl;
  cout << "  cx: " << cx << endl;
  cout << "  cy: " << cy << endl;


  cout << "extrinsic params: " << endl;
  cout << R << endl;
  cout << T << endl;

  double tx,ty,tz;
  tx=T.at<double>(0);
  ty=T.at<double>(1);
  tz=T.at<double>(2);

  double dist=sqrt(tx*tx+ty*ty+tz*tz);
  //cout << "  baseline = " << dist * m_pixel_size << endl;
  cout << "  baseline = " << dist << endl;

  // CALIBRATION QUALITY CHECK
  // because the output fundamental matrix implicitly
  // includes all the output information,
  // we can check the quality of calibration using the
  // epipolar geometry constraint: m2^t*F*m1=0
  double err = 0;
  int npoints = 0;
  vector<Vec3f> lines[2];
  for( i = 0; i < imagecount; i++ )
    {
      int npt = (int)imagePoints[0][i].size();
      Mat imgpt[2];
      for( k = 0; k < 2; k++ )
        {
	  imgpt[k] = Mat(imagePoints[k][i]);
	  undistortPoints(imgpt[k], imgpt[k], cameraMatrix[k], distCoeffs[k], Mat(), cameraMatrix[k]);
	  computeCorrespondEpilines(imgpt[k], k+1, F, lines[k]);
        }
      for( j = 0; j < npt; j++ )
        {
	  double errij = fabs(imagePoints[0][i][j].x*lines[1][j][0] +
			      imagePoints[0][i][j].y*lines[1][j][1] + lines[1][j][2]) +
	    fabs(imagePoints[1][i][j].x*lines[0][j][0] +
		 imagePoints[1][i][j].y*lines[0][j][1] + lines[0][j][2]);
	  err += errij;
        }
      npoints += npt;
    }
  cout << "average reprojection err = " <<  err/npoints << endl;
    
  // save intrinsic parameters
  FileStorage fs("intrinsics.yml", CV_STORAGE_WRITE);
  if( fs.isOpened() )
    {
      fs << "M1" << cameraMatrix[0] << "D1" << distCoeffs[0] <<
	"M2" << cameraMatrix[1] << "D2" << distCoeffs[1];
      fs.release();
    }
  else
    cout << "Error: can not save the intrinsic parameters\n";
    
  Mat R1, R2, P1, P2, Q;
  Rect validRoi[2];
    
#if CV_MINOR_VERSION == 2
#if CV_SUBMINOR_VERSION < 1
  stereoRectify(cameraMatrix[0], distCoeffs[0],
		cameraMatrix[1], distCoeffs[1],
		imageSize, R, T, R1, R2, P1, P2, Q,
		0.6, imageSize, &validRoi[0], &validRoi[1], CALIB_ZERO_DISPARITY);
  cout << "R1 :" << R1 << endl;
  cout << "R2 :" << R2 << endl;
  cout << "P1 :" << P1 << endl;
  cout << "P2 :" << P2 << endl;
  cout << "Q :" << Q << endl;
#else
  stereoRectify(cameraMatrix[0], distCoeffs[0],
		cameraMatrix[1], distCoeffs[1],
		imageSize, R, T, R1, R2, P1, P2, Q,
		CALIB_ZERO_DISPARITY, 0.6, imageSize, &validRoi[0], &validRoi[1]);
#endif
#else
  stereoRectify(cameraMatrix[0], distCoeffs[0],
		cameraMatrix[1], distCoeffs[1],
		imageSize, R, T, R1, R2, P1, P2, Q,
		CALIB_ZERO_DISPARITY, 0.6, imageSize, &validRoi[0], &validRoi[1]);
#endif

  fs.open("extrinsics.yml", CV_STORAGE_WRITE);
  if( fs.isOpened() )
    {
      fs << "R" << R << "T" << T << "R1" << R1 << "R2" << R2 << "P1" << P1 << "P2" << P2 << "Q" << Q;
      fs.release();
    }
  else
    cout << "Error: can not save the intrinsic parameters\n";

  return true;
}


extern "C"
{
 
  void StereoCameraCalibrationInit(RTC::Manager* manager)
  {
    coil::Properties profile(stereocameracalibration_spec);
    manager->registerFactory(profile,
                             RTC::Create<StereoCameraCalibration>,
                             RTC::Delete<StereoCameraCalibration>);
  }
  
};


